// -------- Que fait ce programme ? ---------
/***************************************************
  Ce programme permet de faire fonctionner la serrure debrayable electrique dans des conditions
  experimentales en respectant les conditions de fonctionnement reelles de l'actionneur, cad le moteur
  electrique, lorsqu'il est integre par exemple dans une serrure de porte de chambre d'hotel.
  Le lancement d'un cycle embrayage-debrayage du mecanisme s'effectue par un appui bref sur le
  bouton poussoir BP1.
  Ce programme permet egalement d'effectuer des releves de tension et de courant au niveau du moteur via
  le logiciel Serial Oscilloscope :
  - Oscilloscope1 : voie1 rouge-->Umoteur  (en 1/100eme de V)  voie2 bleu-->Imoteur (en mA) 
  Remarque : pour éviter d'utiliser des "temporisations" logicielles qui pertuberaient le fonctionnement
  de l'affichage des relevés, ce programme utilise un des timers hardware internes au processeur, le timer 1.
  Au lieu d'utiliser une bibliothèque spécifique il a ete decide d'écrire directement dans cet applicatif
  les lignes de code permettant de mettre en oeuvre ce timer.
 ****************************************************/

// /////////////////////////////// 1. Entête déclarative ///////////////////////
// --- Fonctionnalités utilisées ---

// Déclarations de librairies à inclure
#include <stdlib.h>

// --- Déclaration des constantes utiles ---

// --- Déclaration des constantes des broches E/S logiques ---
//const int INFO_SORTIE_V2000C = 4; // info etat sortie V2000C (1L = mot de passe clavier/carte RFID OK)
const int BP1 = 5; // info bouton-poussoir sur PCB
//const int INFO_SORTIE_ZKTECO = 6; // info etat sortie ZKTECO
const int CDE_DIRECTE = 8; // ordre Cde de l'actionneur U positive
const int CDE_INVERSE = 9; // ordre Cde de l'actionneur U negative
//const int BP_PORTE = 10; // info bouton-poussoir sur montant intérieur de la porte
//const int CAPT_POS_PORTE = 11; // info capteur position porte integre a la serrure à pêne dormant (n°1)

// --- Déclaration des constantes des broches analogiques ---
//const int UALIM = A0; // entrée analogique tension générale adaptée
//const int IALIM = A1; // entrée analogique courant général adapté
const int UACT_H = A2 ; // entrée analogique tension +actionneur adaptée
const int UACT_L = A3 ; // entrée analogique tension -actionneur adaptée
const int IACT = A4; // entrée analogique courant actionneur adapté

// --- Déclaration des variables globales ---
volatile unsigned long comptCLIGN1 = 0;
volatile unsigned long currentCLIGN1 = 0;
volatile unsigned long oldCLIGN1 = 0;
boolean X0 = 1; boolean X1 = 0; boolean X2 = 0; boolean X3 = 0;  boolean X4 = 0;

// //////////////////// 2. FONCTION SETUP = Code d'initialisation///////////////////
// La fonction setup() est exécutée en premier et 1 seule fois, au démarrage du programme
void setup()
{
//  pinMode(INFO_SORTIE_V2000C, INPUT_PULLUP); //configuration en entrée logique avec résistance de pull-up
//  pinMode(CAPT_POS_PORTE, INPUT);
  pinMode(BP1, INPUT_PULLUP); //configuration en entrée logique 
  pinMode(CDE_DIRECTE, OUTPUT); //configuration en sortie logique
  pinMode(CDE_INVERSE, OUTPUT); //configuration en sortie logique
  digitalWrite(CDE_DIRECTE, LOW);
  digitalWrite(CDE_INVERSE, HIGH); //debrayabe serrure par cde moteur en inverse pdt 100ms
  delay(100);
  digitalWrite(CDE_INVERSE, LOW);

  Serial.begin(115200); // config baudrate com série
  //while(!Serial);

  // Utilisation du Timer1 pour generer un signal periodique à 500Hz
  noInterrupts(); // desactivation des interruptions
  TCCR1A = 0; TCCR1B = 0; TCNT1 = 0;
  OCR1A = 125; // OCR = Output Compare Register
  // duree période = OCR1A/OCR1Amax (OCR1Amax = 62500) soit 125/62500 = 2ms
  // 62500 correspond à freq horloge / prediviseur = 16000000/256
  TCCR1B |= (1 << WGM12); //affecte 1 au bit WGM12 => CTC mode = Clear timer on compare match
  TCCR1B |= (1 << CS12); //affecte 1 au bit CS12 => prediviseur = 256
  TIMSK1 |= (1 << OCIE1A); // TIMSK1 = Timer/Counter1 Interrupt Mask Register
  //1 << OCIE1A = validation timer compare interrupt
  interrupts(); // activation des interruptions

} // fin de la fonction setup()
// ********************************************************************************

// Fonction s'exécutant par interruption timer toutes les périodes liées à OCR1A
  ISR(TIMER1_COMPA_vect) { // timer compare interrupt service routine
  comptCLIGN1++; currentCLIGN1++; //incrementation à la freq des interruptions 
}

// ////////////////// 3. FONCTION LOOP = Boucle sans fin = coeur du programme ////////////
// la fonction loop() s'exécute sans fin en boucle aussi longtemps que l'Arduino est sous tension

void loop()
{
// Acquisition des grandeurs analogiques alimentation générale + actionneur
  //float valUALIM = analogRead(UALIM);
  //float valIALIM = analogRead(IALIM);
  float valUACT_H = analogRead(UACT_H);
  float valUACT_L = analogRead(UACT_L);
  float valIACT = analogRead(IACT);

// Adaptation des valeurs acquises
  //valUALIM fact_conv=0.2V/V   pour sortie en V => valUALIM*(1/fact_conv)*5/1023
  //valUALIM = valUALIM * 25 / 1023;
  //valIALIM fact_conv=2V/A   pour sortie en A => valIALIM*(1/fact_conv)*5/1023
  //valIALIM = valIALIM * 2.5 / 1023;     //fact conv=2V/A   valIALIM*(1/2)*5/1023
  float valUACT = (valUACT_H - valUACT_L) * 25 / 1023; //idem valUALIM
  valIACT = (valIACT - 520) * 2.5 / 1023; //idem valIALIM avec décalage de -2.5V

  //  Cde séquentielle de la serrure debrayable
  //  boolean PASS_OU_RFID_OK = digitalRead(INFO_SORTIE_V2000C);
  //  boolean POS_PORTE = digitalRead(CAPT_POS_PORTE); // si LOW  la porte est ouverte
  boolean valBP1 = digitalRead(BP1);

  //si BP1 embrayage serrure par cde moteur sens direct pdt 0,1s puis arret
  if (X0 && (valBP1 == LOW)) {
    X1 = 1; X0 = 0; comptCLIGN1 = 0; currentCLIGN1 = 0; oldCLIGN1 = 0;
    digitalWrite(CDE_DIRECTE, HIGH);
  }
  if (X1 && (comptCLIGN1 >= 50 && comptCLIGN1 <= 55)) { //freq comptCLIGN1 = 500Hz
    digitalWrite(CDE_DIRECTE, LOW);
    X2 = 1; X1 = 0; comptCLIGN1 = 0;
  }

  // attente 3s pour permettre d'ouvrir la porte puis cde moteur sens inverse pour debrayer
  if (X2 && (comptCLIGN1 >= 1500 && comptCLIGN1 <= 1520)) {
    digitalWrite(CDE_INVERSE, HIGH);
    X3 = 1; X2 = 0; comptCLIGN1 = 0;
  }

  // debrayage serrure par cde moteur sens direct pdt 0,1s puis arret et retour a X0
  if (X3 && (comptCLIGN1 >= 50 && comptCLIGN1 <= 55)) {
    X4 = 1; X3 = 0; comptCLIGN1 = 0;
    digitalWrite(CDE_INVERSE, LOW);
  }
  
   //attente 0.5s pour affichage
   if (X4 && (comptCLIGN1 >= 250 && comptCLIGN1 <= 255)) {
    X0 = 1; X4 = 0; comptCLIGN1 = 0;
  }
  
//Serial.print("X0 = "); Serial.println(X0);
//Serial.print("X1 = "); Serial.println(X1);
//Serial.print("X2 = "); Serial.println(X2);
//Serial.print("X3 = "); Serial.println(X3);
//Serial.print("comptCLIGN1 = "); Serial.println(comptCLIGN1);
//Serial.println();
//delay(50);

if ((!X0) && (currentCLIGN1 == oldCLIGN1 + 1)) {
    PrintInt((valUACT * 100)*5/12); //Umoteur = UACT - 7V affichage en 1/100 de volt en rouge
    Serial.write(',');  //affichage en 1/100 de volt en bleu
    //PrintInt(comptCLIGN1 * 10);
    PrintInt(valIACT * 1000);
    Serial.write('\r'); // print new line
    oldCLIGN1 = currentCLIGN1;
    }

} // fin de la fonction loop() - le programme recommence au début de la fonction loop sans fin

// Procedure de conversion rapide d'une variable de type "integer" (entier) ou "float" en ASCII
void PrintInt(int i) {
  static const char asciiDigits[10] = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9' };
  div_t n; int print = 0;
  if (i < 0) {
    Serial.write('-');
    i = -i;
  }
  if (i >= 10000) {
    n = div(i, 10000);
    Serial.write(asciiDigits[n.quot]);
    i = n.rem;
    print = 1;
  }
  if (i >= 1000 || print) {
    n = div(i, 1000);
    Serial.write(asciiDigits[n.quot]);
    i = n.rem;
    print = 1;
  }
  if (i >= 100 || print) {
    n = div(i, 100);
    Serial.write(asciiDigits[n.quot]);
    i = n.rem;
    print = 1;
  }
  if (i >= 10 || print) {
    n = div(i, 10);
    Serial.write(asciiDigits[n.quot]);
    i = n.rem;
  }
  Serial.write(asciiDigits[i]);
}
